from osgeo import ogr, osr, gdal

from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.dbRsc import DbRsc
from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.dbSymbols import DbSymbols
from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.db_add_helper import TypeGeometry, DbAttribute

base_name_view="vieq_code_"
name_table_layers_info="layer_info_spr"
def addFeatyreToLayer(layer,fts):
    layer.StartTransaction()
    for ft in fts:
        feature = ogr.Feature(layer.GetLayerDefn())
        geometry=ft['geometry']
        att=ft['attribute']
        for att_ft in att:
            keys=list(att_ft.keys())
            feature.SetField(keys[0],att_ft[keys[0]])
        feature.SetGeometry(geometry)
        layer.CreateFeature(feature)
        feature = None
        geometry = None
    layer.CommitTransaction()
    return

class DbSqliteDataSource:
    def __init__(self):
        self.__connect=None
        self.__path=None
        self.__ds=None
    def connect(self,path_db):
        pass
    def create(self,path_db,name_driver="GPKG"):
        self.__sqlite_dr = ogr.GetDriverByName(name_driver)
        if name_driver=="SQLite":
            dsco = ["SPATIALITE=YES"]
            self.__ds=self.__sqlite_dr.CreateDataSource(path_db, options=dsco)
        else:
            self.__ds = self.__sqlite_dr.CreateDataSource(path_db,)
        if self.__ds is None:
            return False
        self.__path=path_db
        self.__dbRsc=DbRsc(self.__path)
        self.__dbRsc.connect()

        return True
    def open(self,path_db):
        self.__ds=ogr.Open(path_db,update = 1)
        if self.__ds is None:
            return False
        self.__path = path_db
        self.__dbRsc = DbRsc(self.__path)
        self.__dbRsc.connect()
    @property
    def PathDb(self):
        return self.__path

    def createTable(self,name,attribytes,srs_wkt,ogr_type_geometry=ogr.wkbUnknown):
        #self.__dbRsc.close()
        srs = osr.SpatialReference()
        if srs_wkt is None:
            srs.ImportFromEPSG(4326)
        else:
            srs.ImportFromWkt(srs_wkt)
        gdal.UseExceptions()
        layer=self.__ds.CreateLayer(name, srs, geom_type=ogr_type_geometry,options = [ 'OVERWRITE=YES'])

        if attribytes is not None:
            for att in attribytes:
                ogr_def=att.getOgrFieldDefn()
                if ogr_def is None:
                    continue
                layer.CreateField(ogr_def)
        ogeDefMiStyle=DbAttribute.getOgrFieldDefnMiStyle("mi_style")
        layer.CreateField(ogeDefMiStyle)
        layer.SyncToDisk()
        return layer
    def saveTable(self,name,table_info,features,attribytes,srs_wkt,ogr_type_geometry=ogr.wkbUnknown):
        layer=self.createTable(name,attribytes,srs_wkt,ogr_type_geometry)
        addFeatyreToLayer(layer,features)
        layer=None
        sql="insert into catalog_symbols (name,alias,xmin,ymin,xmax,ymax) values(?,?,?,?,?,?)"
        name=name
        alias=name
        if "alias" in table_info:
            alias=table_info['alias']
        values=(name,alias,table_info['xmin'],table_info['ymin'],table_info['xmax'],table_info['ymax'])
        self.__dbRsc.insert(sql,values)
        self.__dbRsc.close(True)
        self.__dbRsc=None
        self.__dbRsc = DbRsc(self.__path)
        self.__dbRsc.connect()

    def deleteSymbolLayer(self,name):
        sql="Delete From "+DbSymbols.name_table+" where name='"+name+"'"
        ret=self.__ds.DeleteLayer(name)
        self.__dbRsc.runSqlNoResult(sql)
        self.__dbRsc.commit()
        jkl=0

    def createCatalog(self):
        #sql='CREATE TABLE [catalog_symbols](  [id] INT PRIMARY KEY UNIQUE, [name] TEXT UNIQUE,  [alias] TEXT,  [xmin] DOUBLE,  [ymin] DOUBLE,  [xmax] DOUBLE,  [ymax] DOUBLE,  [symbol] BLOB)'
        sql = 'CREATE TABLE [catalog_symbols](  [id] INTEGER PRIMARY KEY   AUTOINCREMENT, [name] TEXT UNIQUE,  [alias] TEXT,  [xmin] DOUBLE,  [ymin] DOUBLE,  [xmax] DOUBLE,  [ymax] DOUBLE,  [symbol] BLOB)'
        self.__dbRsc.runSqlNoResult(sql)
    @property
    def DbSqlite(self):
        return self.__dbRsc
    def close(self):
        if self.__ds is not None:
            self.__ds.FlushCache()
            self.__ds==None
            self.__path=None

            if self.__dbRsc is not None:
                self.__dbRsc.close()
                self.__dbRsc=None
            return
        if self.__dbRsc is not None:
            self.__dbRsc.close()
            self.__dbRsc=None

    @staticmethod
    def CreateRscLayer(sp_db,layer):
        sp_db.createTable(layer.Name, layer.Attribute, None, layer.TypeGeometry)

        sql_insert_info_layer="Insert into "+name_table_layers_info+"(id_layer,name,full_name,type_geometry,display_orderIndex) VALUES(?,?,?,?,?)"
        type_geometry=0
        if isinstance(layer.TypeGeometry,int):
            type_geometry=layer.TypeGeometry
        else:
            type_geometry=layer.TypeGeometry.value
        values=(layer.IdLayer,layer.Name,layer.FullName,type_geometry,layer.OrderDisplay)
        sp_db.__dbRsc.insert(sql_insert_info_layer,values)
        sp_db.__dbRsc.commit()
        return
        ''' Create справочники для слоя'''
        dictionary_main_objectId=layer.ObjectIdDictionary
        dbRsc=sp_db.DbSqlite

        #sp_db.close()
        #dbRsc=DbRsc(sp_db.PathDb)
        #isConnect=dbRsc.connect()
        name_add_tab=factoryObjectIdDictionaryDb(dbRsc,layer.Name,dictionary_main_objectId)
        sql="Insert into foreign_key_layers (id_layer, name_children_dict,name_field_base ,name_field_fk,name_filed_children,type_key)  VALUES(?, ?,?,?,?,?)"
        record=(layer.IdLayer,name_add_tab,"objectkey","id_code","name",1)
        dbRsc.insert(sql,record)
        ''' Добавляем справочники на поля '''
        for att in layer.Attribute:
            sp_db.add_dictionary_on_field(layer.IdLayer,att)
        #dbRsc.close()
        dbRsc.commit()
    def add_dictionary_on_field(self,id_layer,att):
        name_view = base_name_view + str(att.InnerCode)
        sql='SELECT name from sqlite_master  WHERE type ="view" and name = "'+name_view+'"'
        records=self.__dbRsc.select(sql)
        if len(records)==0:
            return
        sql="Insert into foreign_key_layers (id_layer, name_children_dict,name_field_base ,name_field_fk,name_filed_children,type_key)  VALUES(?, ?,?,?,?,?)"
        record = (id_layer, name_view, att.Name.lower(),"id_code","name",0)
        self.__dbRsc.insert(sql, record)

    def temp_close(self):
        self.__ds.FlushCache()
        self.__ds=None
    def __reopen(self):
        self.__ds=None

def factoryObjectIdDictionaryDb(dbRsc,name_layer,dictionary_main_objectId):
    name_tab=name_layer+"_dict_object_id"
    sql="create table "+name_tab+" (id_code TEXT,value TEXT, "
    sql=sql+" CONSTRAINT object_key UNIQUE (id_code) )"
    dbRsc.runSqlNoResult(sql)
    sql_insert = "Insert into " + name_tab + " (id_code, value)  VALUES(?, ?)"
    values=[]
    for att in dictionary_main_objectId:
        #rcord=(att.ObjectKey,att.Name)
        #dbRsc.insert(sql_insert,rcord)
        values.append((att.ObjectKey,att.Name))
    #sql_insert="Insert into "+name_tab+" (id_code, value)  VALUES(?, ?)"
    dbRsc.insertMany(sql_insert,values)
    return name_tab









