import axipy
from PySide2.QtCore import QSize, QByteArray, QBuffer, QIODevice, QRect, QPoint, Qt
from PySide2.QtGui import QPainter, QImage
from PySide2.QtSvg import QSvgGenerator
from axipy import Context, Layer, Map, Pnt, Rect, Rectangle, Feature, Style


def saveToSvg(map,rect_out:Rect,path_file,dpi=120):


    generatorSvg = QSvgGenerator()
    size=QSize(rect_out.width*3,rect_out.height*3)
    generatorSvg.setSize(size)
    generatorSvg.setResolution(dpi)
    byte_array=None
    if path_file is None:
        byte_array = QByteArray()
        buffer = QBuffer(byte_array)
        buffer.open(QIODevice.WriteOnly)
        generatorSvg.setOutputDevice(buffer)
    else:
        generatorSvg.setFileName(path_file)
        return None
    generatorSvg.setViewBox(QRect(QPoint(0, 0), size))
    painter_svg = QPainter()
    painter_svg.begin(generatorSvg)
    context_svg = Context(painter_svg)
    #rect_out=Rect(-128,-128,128,128)
    context_svg.rect=rect_out
    map.draw_vector(context_svg)
    return byte_array
def renderTabToSvg(tab,rect,path_out=None,rect_on_bound=True):
    if rect_on_bound:
        rect_geo=Rectangle(rect)
        temp_style=Style.from_mapinfo("Pen (1, 1, 0) Brush (1, 16777215, 16777215)")
        ft=Feature(geometry=rect_geo,style=temp_style)
        tab.insert([ft])
    layer = Layer.create(tab)
    map_source = Map([layer])
    #map_view = axipy.view_manager.create_mapview(map_source)
    #map_view.scale_with_center(rect.width, Pnt(0, 0))
    map = map_source
    svg_bytes=saveToSvg(map,rect,path_out)
    if rect_on_bound:
        tab.undo()
    return svg_bytes
def  renderTabToPng(tab,rect,path_out=None,rect_on_bound=True):
    if rect_on_bound:
        rect_geo=Rectangle(rect)
        temp_style=Style.from_mapinfo("Pen (1, 1, 0) Brush (1, 16777215, 16777215)")
        ft=Feature(geometry=rect_geo,style=temp_style)
        tab.insert([ft])
    layer = Layer.create(tab)
    map_source = Map([layer])
    #map_view = axipy.view_manager.create_mapview(map_source)
    #map_view.scale_with_center(rect.width, Pnt(0, 0))
    map = map_source
    image = QImage(rect.width, rect.height, QImage.Format_ARGB32_Premultiplied)
    image.fill(Qt.transparent)
    painter = QPainter(image)
    context = Context(painter)
    context.rect = rect
    map.draw(context)
    byte_array = None
    if path_out is not None:
        image.save(path_out)
    else:
        byte_array = QByteArray()
        buffer = QBuffer(byte_array)
        buffer.open(QIODevice.WriteOnly)
        image.save(buffer,"PNG")
    if rect_on_bound:
        tab.undo()

    return byte_array



