from PySide2.QtCore import QRect
from axipy import CoordSystem, Map, view_manager, Line, Style, Feature, data_manager, Point

from StyleTools.controls.pointStyleEditor.ui.add_tools.helper_tab import createTempTab

def change_map_view():
    print("Change content")
class MapVectroStyle:
    __style_mi_xy="Pen (1, 6, 0)"
    __style_mi_grid='Symbol (50, 0, 6, "Map Symbols", 0, 0)'

    def __init__(self,table_obj,size=300):
        ''' table_obj может быть str или axipy.Table'''
        max_width=int(size/2)
        str_coordsys='NonEarth Units "mm" Bounds ('+str(-max_width)+','+ str(-max_width)+') ('+str(max_width)+','+str(max_width)+') '
        self.__cur_coordsystem=CoordSystem.from_prj(str_coordsys)
        self.__style_xy = Style.from_mapinfo(self.__style_mi_xy)
        self.__style_grid=Style.from_mapinfo(self.__style_mi_grid)
        self.__initTable(table_obj)
    @staticmethod
    def cretateCs(size):
        max_width = int(size / 2)
        str_coordsys = 'NonEarth Units "mm" Bounds (' + str(-max_width) + ',' + str(-max_width) + ') (' + str(
            max_width) + ',' + str(max_width) + ') '
        coordsystem = CoordSystem.from_prj(str_coordsys)
        return coordsystem


    def __initTable(self,table_obj):
        if isinstance(table_obj,str):
            self.__table_symbol=createTempTab(table_obj,self.__cur_coordsystem,True)
        else:
            self.__table_symbol=table_obj
        self.__tableXY=createTempTab("XY",self.__cur_coordsystem,True)
        self.__tableGrid = createTempTab("GridXY", self.__cur_coordsystem,True)
        self.__createXY()
    def __createMap(self):
        self.__map_base=Map([self.__tableGrid,self.__tableXY,self.__table_symbol])



        self.__map_view=view_manager.create_mapview(self.__map_base)

        ''' Положение и размер окна'''
        rect=QRect(0,0,self.__cur_coordsystem.rect.width*3,self.__cur_coordsystem.rect.height*3)
        self.__map_view.title=self.__table_symbol.name
        self.__map_view.position=rect
        self.__map_view.scene_rect=self.__cur_coordsystem.rect
        layer_editable = None

        for layer in self.__map_view.map.layers:
            if layer.title == self.__table_symbol.name:
                layer_editable = layer
                layer.selectable = True
                layer.visible = True
                continue
            if layer.title == self.__tableXY.name:
                self.__layer_XY=layer

            if layer.title == self.__tableGrid.name:
                self.__layer_Grid=layer

            layer.selectable = False
            layer.visible = False
        self.__setUpLayer(layer_editable.title)
        self.__map_view.editable_layer=layer_editable
        #self.__map_view.scale_with_center(self.__cur_coordsystem.rect.width,self.__cur_coordsystem.rect.center)
    def __setUpLayer(self,name_layer):
        index_layer=None
        for i,lyr in enumerate(self.__map_view.map.layers):
            if lyr.title==name_layer:
                index_layer=i
                break
        if index_layer is None:
            return
        self.__map_view.map.layers.move(index_layer,0)
    def createMapView(self):
        self.__createMap()
        #self.__map_view.scene_changed.connect(change_map_view)
        return self.__map_view
    def viewXY(self,enable=True):
        self.__layer_XY.visible=enable
    def __createXY(self):
        rect=self.__cur_coordsystem.rect
        pt_center=rect.center
        line=Line((rect.xmin,pt_center.y), (pt_center.x,pt_center.y),cs=self.__cur_coordsystem)
        ft=Feature({'id': 0},geometry=line,style=self.__style_xy)
        list_ft=[]
        list_ft.append(ft)
        line = Line((pt_center.x, pt_center.y),(rect.xmax, pt_center.y),  cs=self.__cur_coordsystem)
        ft = Feature({'id': 1}, geometry=line, style=self.__style_xy)
        list_ft.append(ft)
        line = Line((pt_center.x, pt_center.y), (pt_center.x,rect.ymax), cs=self.__cur_coordsystem)
        ft = Feature({'id': 1}, geometry=line, style=self.__style_xy)
        list_ft.append(ft)
        line = Line((pt_center.x, pt_center.y), (pt_center.x, rect.ymin), cs=self.__cur_coordsystem)
        ft = Feature({'id': 1}, geometry=line, style=self.__style_xy)
        list_ft.append(ft)
        self.__tableXY.insert(list_ft)
    def viewGrid(self,step):
        if step is None:
            self.__layer_Grid.visible=False
            #self.__setVisibleLayer(self.__layer_Grid.title, False)
            return
        sql="delete from "+self.__tableGrid.name
        data_manager.query_hidden(sql)
        list_ft=[]

        id=0
        x_curent = self.__cur_coordsystem.rect.center.x
        while x_curent<=self.__cur_coordsystem.rect.xmax:
            y_curent=self.__cur_coordsystem.rect.center.y
            while y_curent<=self.__cur_coordsystem.rect.ymax:
                id=id+1
                pt=Point(x_curent,y_curent,cs=self.__cur_coordsystem)
                ft=Feature({'id':id},geometry=pt,style=self.__style_grid)
                list_ft.append(ft)
                y_curent=y_curent+step
            x_curent=x_curent+step
        x_curent = self.__cur_coordsystem.rect.center.x
        while x_curent <= self.__cur_coordsystem.rect.xmax:
            y_curent = self.__cur_coordsystem.rect.center.y-step
            while y_curent >= self.__cur_coordsystem.rect.ymin:
                id = id + 1
                pt = Point(x_curent, y_curent, cs=self.__cur_coordsystem)
                ft = Feature({'id': id}, geometry=pt, style=self.__style_grid)
                list_ft.append(ft)
                y_curent = y_curent - step
            x_curent = x_curent + step
        x_curent = self.__cur_coordsystem.rect.center.x-step
        while x_curent >= self.__cur_coordsystem.rect.xmin:
            y_curent = self.__cur_coordsystem.rect.center.y
            while y_curent <= self.__cur_coordsystem.rect.ymax:
                id = id + 1
                pt = Point(x_curent, y_curent, cs=self.__cur_coordsystem)
                ft = Feature({'id': id}, geometry=pt, style=self.__style_grid)
                list_ft.append(ft)
                y_curent = y_curent + step
            x_curent = x_curent - step
        x_curent = self.__cur_coordsystem.rect.center.x - step
        while x_curent >= self.__cur_coordsystem.rect.xmin:
            y_curent = self.__cur_coordsystem.rect.center.y-step
            while y_curent >= self.__cur_coordsystem.rect.ymin:
                id = id + 1
                pt = Point(x_curent, y_curent, cs=self.__cur_coordsystem)
                ft = Feature({'id': id}, geometry=pt, style=self.__style_grid)
                list_ft.append(ft)
                y_curent = y_curent - step
            x_curent = x_curent - step
        self.__tableGrid.insert(list_ft)

        self.__layer_Grid.visible = True
        #self.__setVisibleLayer(self.__layer_Grid.title,True)
    def __setVisibleLayer(self,name,visible=True):
        for lyr in self.__map_view.map.layers:
            if lyr.title==name:
                lyr.visible=visible
                return
        return
    @property
    def bound(self):
        return self.__cur_coordsystem.rect
    @property
    def tableSymbol(self):
        return self.__table_symbol
    def closeAll(self):
        self.__tableGrid.rollback()
        self.__tableGrid.close()
        self.__tableXY.rollback()
        self.__tableXY.close()
        self.__table_symbol.close()




