import os

import PySide2
from PySide2 import QtSvg
from PySide2.QtCore import QFile, Qt
from PySide2.QtGui import QPixmap, QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QDialog, QTableWidgetItem, QLabel, QWidget, QHBoxLayout, QVBoxLayout, QSpacerItem, \
    QSizePolicy

from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.dbSymbols import DbSymbols


class DlgSelectStyle(QDialog):
    def __init__(self,db,parent=None):
        self.__ColumnCount=4
        super().__init__(parent)
        self.__dbSymbol=DbSymbols(db)

        self.ui=self.load_ui("dlgSelectStyle.ui",parent)
        self.__initTableSym()
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __initTableSym(self):
        self.ui.tableWidget.setColumnCount(self.__ColumnCount)

        all_symbol=self.__dbSymbol.loadAll()
        all_symbol_count=len(all_symbol)
        row, col = divmod(all_symbol_count, self.__ColumnCount)
        count_row=row
        if col>0:
            count_row=count_row+1
        self.ui.tableWidget.setRowCount(count_row)
        for i,symbol in enumerate(all_symbol):
            row,col = divmod(i,self.__ColumnCount)

            item=self.__createWidgetItemSvg(symbol)
            #self.ui.tableWidget.setItem(row, col, item)
            self.ui.tableWidget.setCellWidget(row, col, item)
        self.ui.tableWidget.resizeRowsToContents()
        jkl=0
    def __createWidgetItem(self,symbol):
        cell_widget = QWidget()
        cell_widget.setStyleSheet('border-style: solid; border-width: 0px; border-color: black;')
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        label_name = QLabel()

        label_name.setAlignment(Qt.AlignCenter)
        label_name.setText(symbol.Name)
        label_name.setFixedHeight(14)
        label = QLabel()

        label.setAlignment(Qt.AlignCenter)
        label.setText(symbol.Name)
        image_sym = QPixmap()
        image_sym.loadFromData(symbol.symbol)
        label.setPixmap(image_sym)
        label.setFixedHeight(32)
        layout.addWidget(label_name)
        layout.addWidget(label)
        cell_widget.setLayout(layout)
        return cell_widget

    '''
        label_widget=QLabel()
        label_widget.setText(symbol.Name)
        image_sym= QPixmap()
        image_sym.loadFromData(symbol.symbol)
        label_widget.setPixmap(image_sym)
        item = QTableWidgetItem(label_widget)
        
        
        image_sym = QPixmap()
        image_sym.loadFromData(symbol.symbol)
        item = QTableWidgetItem(QIcon(image_sym),symbol.Name)
        '''
    def __createWidgetItemSvg(self,symbol):
        version = PySide2.QtCore.__version__

        cell_widget = QWidget()
        cell_widget.setStyleSheet('border-style: solid; border-width: 0px; border-color: black;')
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout_svg = QHBoxLayout()
        layout_svg.setContentsMargins(0, 0, 0, 0)
        horizantSpacer_1 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        horizantSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        verticalSpacer = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Expanding)
        label_name = QLabel()

        label_name.setAlignment(Qt.AlignCenter)
        label_name.setText(symbol.Name)
        label_name.setFixedHeight(14)
        svgWidget = QtSvg.QSvgWidget()
        svgWidget.load(symbol.symbol)
        #svgWidget.setAlignment(Qt.AlignCenter)
        svgWidget.setFixedHeight(44)
        svgWidget.setFixedWidth(44)
        layout_svg.addSpacerItem(horizantSpacer_1)
        layout_svg.addWidget(svgWidget)
        layout_svg.addSpacerItem(horizantSpacer_2)
        layout.addWidget(label_name)

        layout.addLayout(layout_svg)
        layout.addSpacerItem(verticalSpacer)
        #layout.addWidget(svgWidget)
        cell_widget.setLayout(layout)
        return cell_widget
    def show(self):
        self.ui.show()

