import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile, Signal
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget

from StyleTools.controls.pointStyleEditor.ui.add_tools.RenderTools import renderTabToSvg, renderTabToPng
from StyleTools.controls.pointStyleEditor.ui.add_tools.map_style import MapVectroStyle
from StyleTools.controls.pointStyleEditor.ui.wdigetStyleDb import WidgetDbStyle


class MapControlWidget(QWidget):
    end_signal= Signal()
    def __init__(self,cntl_style:WidgetDbStyle,parent=None):
        super().__init__(parent)
        self.__parent_control_style=cntl_style
        self.ui=self.load_ui("widgetMapControl.ui",parent)
        self.ui.ch_b_XY.stateChanged.connect(self.__change_XY)
        self.ui.grbox_grid.toggled.connect(self.__change_status_grid)
        self.ui.spin_grid_step.valueChanged.connect(self.__change_step_grid)
        self.ui.pb_end.clicked.connect(self.__end_edit)
        self.__version_need=axipy.Version.compare(6,0)
        if self.__version_need>=0:
            ''' Версия ниже 6.0
            формат хранения только png 
            '''
            pass

        self.__curent_map=None
        self.__cls_editSymbol=None
        self.__path_out_auto_sync=None
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def createMap(self,name_symbol,size=128):
        if self.__curent_map is not None:
            jkl=0
        self.__cls_editSymbol=MapVectroStyle(name_symbol,size)
        self.__curent_map=self.__cls_editSymbol.createMapView()
        self.__curent_map.title="Символ "+name_symbol.name
        self.__cls_editSymbol.viewXY(True)
    def __change_XY(self,status):
        is_chek=False
        if status>0:
            is_chek=True
        self.__cls_editSymbol.viewXY(is_chek)
    def __change_status_grid(self,status):
        if not status:
            self.__cls_editSymbol.viewGrid(None)
            return
        step=self.ui.spin_grid_step.value()
        self.__cls_editSymbol.viewGrid(step)
    def __change_step_grid(self):
        step = self.ui.spin_grid_step.value()
        self.__cls_editSymbol.viewGrid(step)
    def __end_edit(self):
        ''' Успешное завершение редактирования символа'''
        svg_bytes=None
        tab_symbol=self.__cls_editSymbol.tableSymbol
        name_style=tab_symbol.name
        bound=self.__cls_editSymbol.bound
        if tab_symbol.is_modified:
            ''' Таблица изменена'''
            count_fts=tab_symbol.count()
            if count_fts>0:
                tab_symbol.commit()
                print(self.__version_need)
                if self.__version_need >= 0:
                    ''' Версия ниже 6.0
                    формат хранения только png 
                    '''
                    svg_bytes_qt=renderTabToPng(tab_symbol,bound)
                else:
                    svg_bytes_qt=renderTabToSvg(tab_symbol,bound,None)
                #svg_bytes_qt=renderTabToPng(tab_symbol,bound)
                if svg_bytes_qt is not None:
                    svg_bytes=svg_bytes_qt.data()
        #tab_symbol.close()
        self.__cls_editSymbol.closeAll()
        if svg_bytes is not None:
            self.__parent_control_style.InsertTable(name_style,bound,svg_bytes)
            if self.__path_out_auto_sync is not None:
                path_symbol=Path(self.__path_out_auto_sync)
                path_full_file_symbol=path_symbol.joinpath(name_style+"."+self.__out_format)
                file_path=str(path_full_file_symbol)
                if path_full_file_symbol.exists():
                    os.remove(file_path)
                with open(file_path, "wb") as binary_file:
                    # Write bytes to file
                    binary_file.write(svg_bytes)
        self.end_signal.emit()
        return
    def setPathOut(self,path_out,format):
        self.__path_out_auto_sync=path_out
        self.__out_format=format
